/* This program has been modified from its original form to handle entries to
   a guestlist

     Variable Name     Variable Value
     "recipient"       recipient of guestlist entries
     "maxentries"      maximum number of entries
     "name"            name of person submitting comments
     "email"           email address of person submitting comments
     "comments"        comments

   Modified by Russell Berrett
   Last updated 2/17/96

   Modifications Copyright 1995 SurfUtah.Com.
   Permission granted to copy, modify, and otherwise use the modifications
   to this code in whatever manner you see fit, with no warranty expressed or
   implied.  Please retain this notice; this is the only restriction.  If you
   make any changes to the original code and/or modifications put forth by 
   SurfUtah.Com, plase credit yourself so that SurfUtah.Com is not asked to 
   maintain versions of this modified code that were not developed by 
   SurfUtah.Com.

   SURFUTAH.COM GIVES NO WARRANTY, EXPRESSED OR IMPLIED, FOR THE SOFTWARE 
   AND/OR DOCUMENTATION PROVIDED, INCLUDING, WITHOUT LIMITATION, WARRANTY 
   OF MERCHANTABILITY AND WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE.
*/
   
/* 
        Email handler. This is based on the post_query.c code provided with 
        the NCSA httpd 1.1 package, released to the public domain
        (see their policy, which follows).

        Original portions Copyright 1994 Cold Spring Harbor Labs. 
        Permission granted to copy, modify and otherwise use
        this code in whatever manner you see fit, with no warranty
        expressed or implied. Plesae retain this notice; this is
        the only restriction. We also ask that you credit
        yourself for any changes so we are not asked to maintain
        versions of the code that are no longer recognizable to us. 

        Original portions by Thomas Boutell. 

        Portions developed at the National Center for Supercomputing
        Applications at the University of Illinois at Urbana-Champaign.

        THE UNIVERSITY OF ILLINOIS GIVES NO WARRANTY, EXPRESSED OR IMPLIED,
        FOR THE SOFTWARE AND/OR DOCUMENTATION PROVIDED, INCLUDING, WITHOUT
        LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY OF FITNESS FOR A
        PARTICULAR PURPOSE. */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#define MAX_ENTRIES 1000
#define lineblocksize 100

extern char *getenv(char *e);

typedef struct {
    char *name;
    char *val;
} entry;

typedef char word[2048];


/*-------------------------------------------------------------------------*/
void PrintHeader(char *title)
{
  printf("<html>\n<head>\n");
  printf("<title>%s</title>\n</head>\n", title);
  printf("<body bgcolor=\"#ffffff\" text=\"#000000\">\n");
  printf("<h2>%s</h2>\n<hr>\n", title);
} /* PrintHeader */

/*-------------------------------------------------------------------------*/
void PrintFooter(void)
{
  printf("<p>\n<hr>\n");
  printf("</body>\n</html>\n");
} /* PrintFooter */

/*---------------------------------------------------------------------------
-     NAME:  daiRemoveEOL.
-  PURPOSE:  remove the end of line marker from [thestring].
-      PRE:
-     POST:
-   RETURN:  none.
----------------------------------------------------------------------------*/
static void daiRemoveEOL(char thestring[])
{
  int len;

  len = strlen(thestring);
  if (thestring[len-1] == '\n')
    thestring[len-1] = '\0';
} /* daiRemoveEOL */

/*---------------------------------------------------------------------------
-     NAME:  daGetLine.
-  PURPOSE:   gets the next line in the [stream].
-      PRE:
-     POST:
-   RETURN:  TRUE if successful.
----------------------------------------------------------------------------*/
int daGetLine(char *thestring, FILE *stream)
{
  int success;

  thestring = fgets(thestring, 2048, stream);
  success = (!thestring) ? 0 : 1;
  if (success)
    daiRemoveEOL(thestring);
  return(success);
} /* daGetLine */

/*--------------------------------------------------------------------------*/

/* Removes trailing white space, etc */
void wsremove(char *s);

char *makeword(char *line, char stop);
char *fmakeword(FILE *f, char stop, int *len);
char x2c(char *what);
void unescape_url(char *url);
void plustospace(char *str);


main(int argc, char *argv[]) {
  entry entries[MAX_ENTRIES];
  register int x, m = 0;
  int index, ok = 0;
  int cl, len; 
  unsigned int groupid;
  int recipientid = -1;
  int maxentriesid = -1;
  int nameid = -1;
  int emailid = -1;
  int subjectid = -1;
  int commentsid = -1;
  char filename[512];
  word junk, thestring, tempstring, *curline = NULL;
  int endoffile, linecount, numlines = 0, numblocks = 1;
  int count, tablebegin, maxguests;
  FILE *fp = NULL, *out;
  char buf[81];
  time_t now;

  printf("Content-type: text/html%c%c",10,10);
  if (strcmp(getenv("REQUEST_METHOD"),"POST")) {
    printf("This script should be referenced with a METHOD of POST.\n");
    printf("If you don't understand this, see this ");
    printf("<A HREF=\"http://www.ncsa.uiuc.edu/SDG/Software/Mosaic/Docs/fill-out-forms/overview.html\">forms overview</A>.%c",10);
    exit(1);
  }
  if (strcmp(getenv("CONTENT_TYPE"),"application/x-www-form-urlencoded")) {
    printf("This script can only be used to decode form results. \n");
    exit(1);
  }
  cl = atoi(getenv("CONTENT_LENGTH"));

  for (x=0; cl && (!feof(stdin)); x++) {
    m=x;
    entries[x].val = fmakeword(stdin,'&',&cl);
    plustospace(entries[x].val);
    unescape_url(entries[x].val);
    entries[x].name = makeword(entries[x].val,'=');

      /* set the values of each variable in the array */

    if ((!strcmp(entries[x].name, "name")) &&
             (strcmp(entries[x].val, ""))) {
      nameid = x;
    } 
    else if ((!strcmp(entries[x].name, "recipient")) &&
             (strcmp(entries[x].val, ""))) {
      recipientid = x;
    } 
    else if ((!strcmp(entries[x].name, "email")) &&
             (strcmp(entries[x].val, ""))) {
      emailid = x;
    } 
    else if ((!strcmp(entries[x].name, "comments")) &&
             (strcmp(entries[x].val, ""))) {
      commentsid = x;
    } 
    else if ((!strcmp(entries[x].name, "maxentries")) &&
             (strcmp(entries[x].val, ""))) {
      maxentriesid = x;
    } 
  }

    /* check to see if all the infomation was entered properly, 
       if not... reject the request */

  if (nameid == -1) {
    PrintHeader("Entry Rejected");
    printf("<p><font size=\"+1\">\n");
    printf("Your request to add an entry to the guestlist has been\n");
    printf("rejected due to insufficient information.\n<p>\n");
    printf("To properly send your comments,\n");
    printf("please fill out: \n");
    printf("<ul>\n");
    printf("<li>Your name\n");
    printf("</ul></font>\n");
    PrintFooter();
    return(0);
  }

    /* a simple profanity check, add more if necessary */

  if ((commentsid != -1) &&
      (strstr(entries[commentsid].val, "fuck") ||
       strstr(entries[commentsid].val, "shit") ||
       strstr(entries[commentsid].val, "bastard") ||
       strstr(entries[commentsid].val, "bitch"))) {
    PrintHeader("Entry Rejected");
    printf("<p><font size=\"+1\">\n");
    printf("Your request to add an entry to the guestlist has been\n");
    printf("rejected due to improper language.\n<p>\n");
    printf("</font>\n");
    PrintFooter();
    return(0);
  }

  if ((recipientid != -1) && (emailid != -1)) {
    sprintf(buf, "/bin/sendmail %s", entries[recipientid].val);
    out = popen(buf, "w");
    fprintf(out, "To: %s\n", entries[recipientid].val);
    fprintf(out, "Subject: Guestlist Entry\n");
    fprintf(out, "From: %s\n\n", entries[emailid].val);
    fprintf(out, "Reply-To: %s\n\n", entries[emailid].val);
    fprintf(out, "*---------------------------------------------------*\n");
    fprintf(out, "| ------------------------------------------------- |\n");
    fprintf(out, "|     This message sent from the Guestlist page     |\n");
    fprintf(out, "| ------------------------------------------------- |\n");
    fprintf(out, "*---------------------------------------------------*\n");
    fprintf(out, "\n");
    fprintf(out, "Name :  %s\n\n", entries[nameid].val);
    if (commentsid != -1)
      fprintf(out, "Comments :  %s\n\n", entries[commentsid].val);
    fprintf(out, "\n");
    pclose(out);
  }

  sprintf(filename, "%s", getenv("PATH_TRANSLATED"));
  fp = NULL;
  fp = fopen(filename, "r");
  if (!fp) {
    len = 0;
    while ((len < 100) && !fp) {
      len++;
      fp = fopen(filename, "r");
    }   
  }
  if (fp) {
      /* first disable any html tags in the comments, name, or email */
    count = 0;
    len = strlen(entries[nameid].val);
    for (index = 0; index < len; index++) {
      if (entries[nameid].val[index] == '<') {
        junk[count++] = '&';
        junk[count++] = 'l';
        junk[count++] = 't';
        junk[count++] = ' ';
      }
      else if (entries[nameid].val[index] == '>') {
        junk[count++] = '&';
        junk[count++] = 'g';
        junk[count++] = 't';
        junk[count++] = ' ';
      }
      else
        junk[count++] = entries[nameid].val[index];
    }
    junk[count++] = '\0';
    strcpy(entries[nameid].val, junk);
    if (emailid != -1) {
      count = 0;
      len = strlen(entries[emailid].val);
      for (index = 0; index < len; index++) {
        if (entries[emailid].val[index] == '<') {
          junk[count++] = '&';
          junk[count++] = 'l';
          junk[count++] = 't';
          junk[count++] = ' ';
        }
        else if (entries[emailid].val[index] == '>') {
          junk[count++] = '&';
          junk[count++] = 'g';
          junk[count++] = 't';
          junk[count++] = ' ';
        }
        else
          junk[count++] = entries[emailid].val[index];
      }
      junk[count++] = '\0';
      strcpy(entries[emailid].val, junk);
    }
    if (commentsid != -1) {
      count = 0;
      len = strlen(entries[commentsid].val);
      for (index = 0; index < len; index++) {
        if (entries[commentsid].val[index] == '<') {
          junk[count++] = '&';
          junk[count++] = 'l';
          junk[count++] = 't';
          junk[count++] = ' ';
        }
        else if (entries[commentsid].val[index] == '>') {
          junk[count++] = '&';
          junk[count++] = 'g';
          junk[count++] = 't';
          junk[count++] = ' ';
        }
        else
          junk[count++] = entries[commentsid].val[index];
      }
      junk[count++] = '\0';
      strcpy(entries[commentsid].val, junk);
    }

    sscanf(entries[maxentriesid].val, "%d", &maxguests);
    count = tablebegin = 0;
    curline = (word *)malloc(lineblocksize * sizeof(word));
    endoffile = !daGetLine(thestring, fp);
    while (!endoffile) {
      if (strstr(thestring, "<!-- __GUESTLIST_BEGIN__ -->")) {
        tablebegin = 1;
	strcpy(curline[numlines++], thestring);
	sprintf(thestring, "<hr><p align=center>Host: %s<br>", 
		getenv("REMOTE_HOST"));
	now = time(NULL);
	strftime(tempstring, 2048, "%B, %d %Y &#160; &#160; &#160; %r CET", 
		 localtime(&now)); 
	strcat(thestring, tempstring);
	strcat(thestring, "<br>");
	if (emailid == -1)
          sprintf(tempstring, "%s<br>", entries[nameid].val);
        else
          sprintf(tempstring, "<a href = \"mailto:%s\">%s</a><br>", 
                  entries[emailid].val, entries[nameid].val);
        strcat(thestring, tempstring);
	sprintf(tempstring, "%s<br></p>", 
                ((commentsid == -1) ? "no comments" : 
                                      entries[commentsid].val));
        strcat(thestring, tempstring);
	count++;
	strcpy(curline[numlines++], thestring);
      }
      else if (strstr(thestring, "<!-- __GUESTLIST_END__ -->")) {
	tablebegin = 0;
	strcpy(curline[numlines++], thestring);
      }
      else {
	if (tablebegin) {
	  if ((maxguests <= 0) || (count < maxguests)) {
	    count++;
	    strcpy(curline[numlines++], thestring);
	  }
	}
	else {
	  strcpy(curline[numlines++], thestring);
        }
      }
      endoffile = !daGetLine(thestring, fp);
      if (numlines >= (numblocks * lineblocksize)) {
        numblocks++;
        curline = (word *)realloc(curline, (numblocks * lineblocksize *
                                            sizeof(word)));
      }
    }
    fclose(fp);
    fp = NULL;
    fp = fopen(filename, "w");
    if (!fp) {
      len = 0;
      while (!fp && (len < 100)) {
	len = 100;
	fp = fopen(filename, "w");
      }
    }
    if (!fp) {
      printf("Cannot open file \"%s\": \n", filename); 
      printf("The file does not exist or \n");
      printf("cannot be opened (check permissions)\n");
      return(0);
    }
    for (index = 0; index < numlines; index++)
      fprintf(fp, "%s\n", curline[index]);
    fprintf(fp, "\0");
    fclose(fp);
    if (curline) {
      free(curline);
      curline = NULL;
    }
  }
  else {
    printf("Cannot open file \"%s\": \n", filename); 
    printf("The file does not exist or \n");
    printf("cannot be opened (check permissions)\n");
    return(0);
  }

    /* echo back the input */

  PrintHeader("Entry Accepted");
  printf("<p><font size=\"+1\">\n<table border=0 cellpadding=5 cellspacing=10>");
  printf("<tr><td><i>Your Name :</i></td><td>  %s", entries[nameid].val); 
  if (emailid != -1) 
    printf("</td></tr><tr><td><i>Your Email :</i></td><td>  %s ", entries[emailid].val); 
  if (commentsid != -1)
    printf("</td></tr><tr><td><i>Comments :</i></td><td>  %s", entries[commentsid].val); 
  printf("</td></tr></table><p>\n");
  sprintf(filename, "%s", getenv("PATH_INFO"));
  printf("<a href = \"%s\">View your entry on the Guestlist</a>", filename);
  printf("</font>\n");
  PrintFooter();
}

/* Removes trailing white space, etc */
void wsremove(char *s) {
  while(1) {
    int l = strlen(s);
    if (!l) {
      break;
    }
    if (isspace(s[l-1])) {
      s[l-1] = '\0';
    } 
    else {
      break;
    }
  }
} 
